@@ -32,9 +32,10 @@ const common = require('_tls_common');
 const { StreamWrap } = require('_stream_wrap');
 const { Buffer } = require('buffer');
 const debug = util.debuglog('tls');
-const tls_wrap = process.binding('tls_wrap');
-const { TCP, constants: TCPConstants } = process.binding('tcp_wrap');
-const { Pipe, constants: PipeConstants } = process.binding('pipe_wrap');
+const { TCP, constants: TCPConstants } = internalBinding('tcp_wrap');
+const tls_wrap = internalBinding('tls_wrap');
+const { Pipe, constants: PipeConstants } = internalBinding('pipe_wrap');
+const { owner_symbol } = require('internal/async_hooks').symbols;
 const {
   SecureContext: NativeSecureContext
 } = process.binding('crypto');
@@ -50,6 +51,7 @@ const {
   ERR_TLS_SESSION_ATTACK,
   ERR_TLS_SNI_FROM_SERVER
 } = require('internal/errors').codes;
+const { validateString } = require('internal/validators');
 const kConnectOptions = Symbol('connect-options');
 const kDisableRenegotiation = Symbol('disable-renegotiation');
 const kErrorEmitted = Symbol('error-emitted');
@@ -63,7 +65,8 @@ function onhandshakestart(now) {
   debug('onhandshakestart');
 
   const { lastHandshakeTime } = this;
-  assert(now >= lastHandshakeTime);
+  assert(now >= lastHandshakeTime,
+         `now (${now}) < lastHandshakeTime (${lastHandshakeTime})`);
 
   this.lastHandshakeTime = now;
 
@@ -76,7 +79,7 @@ function onhandshakestart(now) {
   else
     this.handshakes++;
 
-  const { owner } = this;
+  const owner = this[owner_symbol];
   if (this.handshakes > tls.CLIENT_RENEG_LIMIT) {
     owner._emitTLSError(new ERR_TLS_SESSION_ATTACK());
     return;
@@ -89,7 +92,7 @@ function onhandshakestart(now) {
 function onhandshakedone() {
   debug('onhandshakedone');
 
-  const owner = this.owner;
+  const owner = this[owner_symbol];
 
   // `newSession` callback wasn't called yet
   if (owner._newSessionPending) {
@@ -102,7 +105,7 @@ function onhandshakedone() {
 
 
 function loadSession(hello) {
-  const owner = this.owner;
+  const owner = this[owner_symbol];
 
   var once = false;
   function onSession(err, session) {
@@ -130,7 +133,7 @@ function loadSession(hello) {
 
 
 function loadSNI(info) {
-  const owner = this.owner;
+  const owner = this[owner_symbol];
   const servername = info.servername;
   if (!servername || !owner._SNICallback)
     return requestOCSP(owner, info);
@@ -212,7 +215,7 @@ function requestOCSPDone(socket) {
 
 
 function onnewsession(key, session) {
-  const owner = this.owner;
+  const owner = this[owner_symbol];
 
   if (!owner.server)
     return;
@@ -241,15 +244,17 @@ function onnewsession(key, session) {
 
 
 function onocspresponse(resp) {
-  this.owner.emit('OCSPResponse', resp);
+  this[owner_symbol].emit('OCSPResponse', resp);
 }
 
 function onerror(err) {
-  const owner = this.owner;
+  const owner = this[owner_symbol];
 
-  if (owner._writableState.errorEmitted)
+  if (owner._hadError)
     return;
 
+  owner._hadError = true;
+
   // Destroy socket if error happened before handshake's finish
   if (!owner._secureEstablished) {
     // When handshake fails control is not yet released,
@@ -264,8 +269,6 @@ function onerror(err) {
     // Throw error
     owner._emitTLSError(err);
   }
-
-  owner._writableState.errorEmitted = true;
 }
 
 function initRead(tls, wrapped) {
@@ -307,10 +310,12 @@ function TLSSocket(socket, opts) {
 
   // Wrap plain JS Stream into StreamWrap
   var wrap;
-  if ((socket instanceof net.Socket && socket._handle) || !socket)
+  if ((socket instanceof net.Socket && socket._handle) || !socket) {
     wrap = socket;
-  else
+  } else {
     wrap = new StreamWrap(socket);
+    wrap.once('close', () => this.destroy());
+  }
 
   // Just a documented property to make secure sockets
   // distinguishable from regular ones.
@@ -364,9 +369,9 @@ for (var n = 0; n < proxiedMethods.length; n++) {
 
 tls_wrap.TLSWrap.prototype.close = function close(cb) {
   let ssl;
-  if (this.owner) {
-    ssl = this.owner.ssl;
-    this.owner.ssl = null;
+  if (this[owner_symbol]) {
+    ssl = this[owner_symbol].ssl;
+    this[owner_symbol].ssl = null;
   }
 
   // Invoke `destroySSL` on close to clean up possibly pending write requests
@@ -405,7 +410,7 @@ TLSSocket.prototype._wrapHandle = function(wrap) {
     handle = options.pipe ?
       new Pipe(PipeConstants.SOCKET) :
       new TCP(TCPConstants.SOCKET);
-    handle.owner = this;
+    handle[owner_symbol] = this;
   }
 
   // Wrap socket's handle
@@ -645,9 +650,7 @@ TLSSocket.prototype._start = function() {
 };
 
 TLSSocket.prototype.setServername = function(name) {
-  if (typeof name !== 'string') {
-    throw new ERR_INVALID_ARG_TYPE('name', 'string', name);
-  }
+  validateString(name, 'name');
 
   if (this._tlsOptions.isServer) {
     throw new ERR_TLS_SNI_FROM_SERVER();
@@ -874,6 +877,8 @@ function Server(options, listener) {
     ciphers: this.ciphers,
     ecdhCurve: this.ecdhCurve,
     dhparam: this.dhparam,
+    minVersion: this.minVersion,
+    maxVersion: this.maxVersion,
     secureProtocol: this.secureProtocol,
     secureOptions: this.secureOptions,
     honorCipherOrder: this.honorCipherOrder,
@@ -945,6 +950,8 @@ Server.prototype.setOptions = function(options) {
   if (options.clientCertEngine)
     this.clientCertEngine = options.clientCertEngine;
   if (options.ca) this.ca = options.ca;
+  if (options.minVersion) this.minVersion = options.minVersion;
+  if (options.maxVersion) this.maxVersion = options.maxVersion;
   if (options.secureProtocol) this.secureProtocol = options.secureProtocol;
   if (options.crl) this.crl = options.crl;
   if (options.ciphers) this.ciphers = options.ciphers;
@@ -1119,7 +1126,7 @@ exports.connect = function connect(...args) {
 
   const context = options.secureContext || tls.createSecureContext(options);
 
-  var socket = new TLSSocket(options.socket, {
+  var tlssock = new TLSSocket(options.socket, {
     pipe: !!options.path,
     secureContext: context,
     isServer: false,
@@ -1130,36 +1137,44 @@ exports.connect = function connect(...args) {
     requestOCSP: options.requestOCSP
   });
 
-  socket[kConnectOptions] = options;
+  tlssock[kConnectOptions] = options;
 
   if (cb)
-    socket.once('secureConnect', cb);
+    tlssock.once('secureConnect', cb);
 
   if (!options.socket) {
+    // If user provided the socket, its their responsibility to manage its
+    // connectivity. If we created one internally, we connect it.
     const connectOpt = {
       path: options.path,
       port: options.port,
       host: options.host,
       family: options.family,
       localAddress: options.localAddress,
+      localPort: options.localPort,
       lookup: options.lookup
     };
-    socket.connect(connectOpt, socket._start);
+
+    if (options.timeout) {
+      tlssock.setTimeout(options.timeout);
+    }
+
+    tlssock.connect(connectOpt, tlssock._start);
   }
 
-  socket._releaseControl();
+  tlssock._releaseControl();
 
   if (options.session)
-    socket.setSession(options.session);
+    tlssock.setSession(options.session);
 
   if (options.servername)
-    socket.setServername(options.servername);
+    tlssock.setServername(options.servername);
 
   if (options.socket)
-    socket._start();
+    tlssock._start();
 
-  socket.on('secure', onConnectSecure);
-  socket.once('end', onConnectEnd);
+  tlssock.on('secure', onConnectSecure);
+  tlssock.once('end', onConnectEnd);
 
-  return socket;
+  return tlssock;
 };
